"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessError = void 0;
exports.calcDuration = calcDuration;
exports.executeProcess = executeProcess;
const node_child_process_1 = require("node:child_process");
function calcDuration(start, stop) {
    return Math.round((stop ?? performance.now()) - start);
}
/**
 * Error class for process errors.
 * Contains additional information about the process result.
 * @category Error
 * @public
 * @class
 * @extends Error
 * @example
 * const result = await executeProcess({})
 * .catch((error) => {
 *   if (error instanceof ProcessError) {
 *   console.error(error.code);
 *   console.error(error.stderr);
 *   console.error(error.stdout);
 *   }
 * });
 *
 */
class ProcessError extends Error {
    code;
    stderr;
    stdout;
    constructor(result) {
        super(result.stderr);
        this.code = result.code;
        this.stderr = result.stderr;
        this.stdout = result.stdout;
    }
}
exports.ProcessError = ProcessError;
/**
 * Executes a process and returns a promise with the result as `ProcessResult`.
 *
 * @example
 *
 * // sync process execution
 * const result = await executeProcess({
 *  command: 'node',
 *  args: ['--version']
 * });
 *
 * console.info(result);
 *
 * // async process execution
 * const result = await executeProcess({
 *    command: 'node',
 *    args: ['download-data.js'],
 *    observer: {
 *      onStdout: updateProgress,
 *      error: handleError,
 *      complete: cleanLogs,
 *    }
 * });
 *
 * console.info(result);
 *
 * @param cfg - see {@link ProcessConfig}
 */
function executeProcess(cfg) {
    const { observer, cwd, command, args, ignoreExitCode = false } = cfg;
    const { onStdout, onError, onComplete } = observer ?? {};
    const date = new Date().toISOString();
    const start = performance.now();
    return new Promise((resolve, reject) => {
        // shell:true tells Windows to use shell command for spawning a child process
        const process = (0, node_child_process_1.spawn)(command, args, { cwd, shell: true });
        // eslint-disable-next-line functional/no-let
        let stdout = '';
        // eslint-disable-next-line functional/no-let
        let stderr = '';
        process.stdout.on('data', data => {
            stdout += String(data);
            onStdout?.(String(data));
        });
        process.stderr.on('data', data => {
            stderr += String(data);
        });
        process.on('error', err => {
            stderr += err.toString();
        });
        process.on('close', code => {
            const timings = { date, duration: calcDuration(start) };
            if (code === 0 || ignoreExitCode) {
                onComplete?.();
                resolve({ code, stdout, stderr, ...timings });
            }
            else {
                const errorMsg = new ProcessError({ code, stdout, stderr, ...timings });
                onError?.(errorMsg);
                reject(errorMsg);
            }
        });
    });
}
//# sourceMappingURL=execute-process.js.map